//--------------------------------------------------------------------
//
// DISCOUNT: Display-Controller
//
// Stellt einen 8-Bit-Speicher auf einem Video-Monitor monochrom dar
// (Aufloesung 640x192, Bildfrequenz 50 Hz).
//
//--------------------------------------------------------------------

//--------------------------------------------------------------------
//
// Modul discount_ram - Version mit variabel programmierbarer Aufloesung
//                      und beschreibbarem Videospeicher
//
// Display-Controller (Cathode-Ray-Tube-Controller)
//
//--------------------------------------------------------------------

`include "discount_defs.v"

module discount_ram (PIXEL, CSYNC, HSYNC, VSYNC,
                     CLOCK, NRST, PADDR, WE, PDATA);

  // Ausgaenge
  output            PIXEL,  // Bildpunkt (0: dunkel, 1: hell)
                    CSYNC,  // kombinierte Synchronisation
                            //   (HSYNC und VSYNC)
                    HSYNC,  // horizontale (Zeilen-)Synchronisation
                            //   (1-aktiv)
                    VSYNC;  // vertikale (Bild-)Synchronisation
                            //   (1-aktiv)

  // Eingaenge
  input             CLOCK,  // Takt (einphasig)
                    NRST,   // Initialisierung (0-aktiv)
                    WE;     // Write-Enable fuer Programmierbus
  input  [`Asz-1:0] PADDR;  // Programmier-Adressbus

  // Bidirektional
  inout  [`Rsz-1:0] PDATA;  // Programmier-Datenbus

  // Port-Typen
  wire   [`Asz-1:0] PADDR;
  wire              PIXEL,
                    CSYNC,
                    HSYNC,
                    VSYNC,
                    CLOCK,
                    NRST,
                    WE;
  wire   [`Rsz-1:0] PDATA;

  // interne Variablen
  wire              ADRIE,    // Freigabe: Bilddaten uebernehmen und
                              //   Bildadresse erhoehen
                              //   (address increment enable)
                    NXTAD,    // Bildadresse hochzaehlen
                    NXTLN;    // Zeile hochzaehlen
  wire   [`Asz-1:0] ADDR;     // Adresse von MEMACC
  wire   [`Asz-1:0] RAM_ADDR; // Adresse fuer Bildspeicher
  wire   [`Dsz-1:0] DATA;     // Daten aus dem Bildspeicher

  //
  // kombinierte Synchronisation
  //
  assign CSYNC = HSYNC ^ VSYNC;

  //
  // Bildspeicheradresse
  // entweder vom Programmierbus (neuen Inhalt schreiben)
  // oder von MEMACC (Bilddaten darstellen)
  //
  assign RAM_ADDR = (WE & PADDR[14]) ? PADDR : ADDR;

  //
  // Instanzen
  //
  hcount_ram HCOUNT (NXTAD, NXTLN, HSYNC, CLOCK, NRST, PADDR, PDATA, WE);
  vcount_ram VCOUNT (ADRIE, VSYNC, NXTLN, CLOCK, NRST, PADDR, PDATA, WE);
  memacc_ram MEMACC (ADDR,  PIXEL, DATA,  ADRIE, NXTAD,
                     CLOCK, NRST, PADDR, PDATA, WE);
  ram        RAM    (CLOCK, NRST, RAM_ADDR[13:0],
                     PDATA[`Bsz-1:0], WE & PADDR[14], DATA);

endmodule  // discount_ram
