//--------------------------------------------------------------------
//
// Modul hcount_ram - Version mit variabel programmierbarer Aufloesung
//                    und beschreibbarem Videospeicher
//
// Timing der Zeilen; Steuersignale fuer Adressen, Bildpunkte,
// horizontale Synchronisation und Timing des Bildes
//
//--------------------------------------------------------------------
//
// Schnittstelle
//
// NXTAD  im naechsten Takt Bildspeicherdaten uebernehmen,
//        zur naechsten Bildspeicheradresse weiterschalten (1-aktiv)
// NXTLN  im naechsten Takt Zeile hochzaehlen (1-aktiv)
// HSYNC  horizontale (Zeilen-)Synchronisation (1-aktiv)
// CLOCK  Takt
// NRST   Initialisierung (0-aktiv)
// PADDR  Programmier-Adressbus
// DATA   Programmier-Datenbus
// WE     Write-Enable fuer Programmierbus
//
//--------------------------------------------------------------------
//
// Timing und Arbeitsperiode von hcount:
//
// H1 Takte  linker Bildrand
//            HSYNC=0, NXTAD=0, NTXLN=0
// H  Takte  Bildpunkte
//            HSYNC=0, NXTLN=0, NXTAD wird alle 8 Takte gesetzt
// H2 Takte  rechter Bildrand
//            HSYNC=0, NXTAD=0, NTXLN=0
// H3 Takte  horizontale Synchronisation
//            HSYNC=1, NXTAD=0, NXTLN wird im letzten Takt 1
//
//--------------------------------------------------------------------

`include "discount_defs.v"

module hcount_ram (NXTAD, NXTLN, HSYNC, CLOCK, NRST, PADDR, DATA, WE);

  // Ausgaenge
  output NXTAD,      // Bildadresse hochzaehlen
         NXTLN,      // Zeile hochzaehlen
         HSYNC;      // Zeilensynchronisation

  // Eingaenge
  input            CLOCK,      // Takt
                   NRST,       // Initialisierung (0-aktiv)
                   WE;         // Write-Enable
  input [`Asz-1:0] PADDR;      // Adressbus

  // Bidirektional
  inout [`Rsz-1:0] DATA;       // Datenbus

  // Port-Typen
  reg             NXTAD,
                  NXTLN,
                  HSYNC;
  reg  [`Rsz-1:0] DATA_OUT;
  wire            CLOCK,
                  NRST;
  wire [`Asz-1:0] PADDR;

  // interne Variablen
  reg [`Rsz-1:0] HCNT;    // Taktzaehler innerhalb einer Zeile
  reg [1:0]      HPHASE;  // Phasen horizontal:
                          //   0 linker Bildrand
                          //   1 Bildpunkte
                          //   2 rechter Bildrand
                          //   3 horizontale Synchronisation
  reg [`Rsz-1:0] H1, // Register (variable Aufloesung)
                 H,
                 H2,
                 H3,
                 Hline;
  reg [`Rsz-1:0] Hline_temp,  // Temporaere Flip-Flops
                 Hline_temp2; // fuer Pipeline
  reg            ADDRESSED;   // Gueltige Adresse im vorigen Takt


  //
  // Bidirektionalen Bus asynchron treiben
  // Daten einen Takt verzoegert zu Adressen -> ADDRESSED
  //
  assign DATA = (WE | ~ADDRESSED) ? `Rsz'bz : DATA_OUT;

  //
  // Register synchron auslesen
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0) begin                       // Reset, magic DisCount
      DATA_OUT <= `Rsz'hDC;
      ADDRESSED <= 0;
    end
    else begin
      case (PADDR[2:0])
        0: DATA_OUT <= H1;
        1: DATA_OUT <= H;
        2: DATA_OUT <= H2;
        3: DATA_OUT <= H3;
        default: DATA_OUT <= `Rsz'hBA;         // magic Bad Address
      endcase
      ADDRESSED <= ~PADDR[2] & ~PADDR[14];
    end
  end

  //
  // Register schreiben
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0) begin                       // Alle Register auf
      H1 <= `H1;                               // Default-Werte setzen
      H  <= `H;
      H2 <= `H2;
      H3 <= `H3;
      Hline       <= `H1+`H+`H2+`H3;
      Hline_temp  <= `H1+`H;
      Hline_temp2 <= `H1+`H+`H2;
    end
    else begin
      Hline_temp <= H1 + H;                    // Summe in Pipeline
      Hline_temp2 <= Hline_temp + H2;          // berechnen
      Hline <= Hline_temp2 + H3;
      if (WE)                                  // Es wird geschrieben
        case ({PADDR[14],PADDR[2:0]})          // Daten uebernehmen
          0: H1 <= DATA;
          1: H  <= DATA;
          2: H2 <= DATA;
          3: H3 <= DATA;
        endcase
    end
  end

  //
  // Taktzaehler aktualisieren
  // (Periode Hline, Bereich 0..Hline-1)
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0)         HCNT <= `Hline-1;    // Reset
    else
      if (HCNT == Hline-1) HCNT <= 0;           // neue Zeile
      else                 HCNT <= HCNT+1;
  end

  //
  // Phasenzaehler aktualisieren
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0) HPHASE <= 3;        // Reset
    else
      case (HCNT)
        0:         HPHASE <= 0;        // linker Bildrand
        H1:        HPHASE <= HPHASE+1; // Bildpunkte
        H1+H:      HPHASE <= HPHASE+1; // rechter Bildrand
        H1+H+H2:   HPHASE <= HPHASE+1; // Bildsynchronisation
      endcase
  end

  //
  // horizontale Synchronisation aktualisieren;
  // HSYNC wird ueber ein Register ausgegeben, um die Verzoegerung
  // auf NXTAD von einem Takt zu kompensieren und so das Timing nach
  // aussen einzuhalten
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0)     HSYNC <= 1;   // Reset
    else
      if (HPHASE == 3) HSYNC <= 1;
      else             HSYNC <= 0;
  end

  //
  // NXTAD und NXTLN aktualisieren
  //
  always @(HCNT or HPHASE) begin
    if ((HCNT % `Bsz == 1 )                     // neues Byte
      && (HPHASE == 1))               NXTAD = 1;
    else                              NXTAD = 0;
    if ((HCNT == 0) && (HPHASE == 3)) NXTLN = 1;
    else                              NXTLN = 0;
  end

endmodule // hcount_ram
