//--------------------------------------------------------------------
//
// Modul memacc_ram - Version mit variabel programmierbarer Aufloesung
//                    und beschreibbarem Videospeicher
//
// Adressen fuer den Bildspeicher generieren, Bilddaten als Bytes
// lesen und als Punkte ausgeben
//
//--------------------------------------------------------------------
//
// Schnittstelle
//
//  ADDR   Adresse fuer Bildspeicher
//  PIXEL  Bildpunkt (0: dunkel, 1: hell)
//  DATA   Daten aus dem Bildspeicher
//  ADRIE  Freigabe: Bilddaten uebernehmen und Bildadresse erhoehen
//         (address increment enable)
//  NXTAD  Daten uebernehmen, Bildadresse hochzaehlen
//  CLOCK  Takt
//  NRST   Initialisierung (0-aktiv)
//  PADDR  Programmier-Adressbus
//  PDATA  Programmier-Datenbus
//  WE     Write-Enable fuer Programmierbus
//
//--------------------------------------------------------------------
//
// Timing und Funktion:
//
// Bei einer positiven Taktflanke und ADRIE=NXTAD=1 werden das
// Bildpunkt-Schieberegister mit den momentanen Bildspeicherdaten
// geladen und die Bildspeicheradresse erhoeht. Letztere liegt im
// Bereich von 0 bis `Msz, worauf wieder 0 folgt.
// Ist ADRIE oder NXTAD 0, wird das Bildpunkt-Schieberegister
// weitergeschoben und beim niedrigstwertigen Bit mit 0 gefuellt.
//
//--------------------------------------------------------------------

`include "discount_defs.v"

module memacc_ram (ADDR, PIXEL, DATA, ADRIE, NXTAD,
                   CLOCK, NRST, PADDR, PDATA, WE);

  // Ausgaenge
  output [`Asz-1:0] ADDR;   // Adresse fuer Bildspeicher
  output            PIXEL;  // Bildpunkt

  // Eingaenge
  input  [`Dsz-1:0] DATA;   // Daten aus dem Bildspeicher
  input             ADRIE,  // Freigabe: Bildadresse erhoehen
                    NXTAD,  // Bildadresse hochzaehlen
                    CLOCK,  // Takt
                    NRST,   // Initialisierung (0-aktiv)
                    WE;     // Write-Enable fuer Programmierbus
  input [`Asz-1:0]  PADDR;  // Programmieradressbus

  // Bidirektional
  inout [`Rsz-1:0]  PDATA;  // Programmierdatenbus

  // Port-Typen
  reg    [`Asz-1:0] ADDR;
  wire   [`Dsz-1:0] DATA;
  wire              PIXEL,
                    ADRIE,
                    NXTAD,
                    CLOCK,
                    NRST,
                    WE;
  wire   [`Asz-1:0] PADDR;
  wire   [`Rsz-1:0] PDATA;

  // interne Variablen
  reg    [`Dsz-1:0] PIXSR;  // Bildpunkt-Schieberegister
  reg    [`Rsz-1:0] H,      // Register (variable Aufloesung)
                    V;
  reg    [`Asz-1:0] Msz;

  //
  // Bildpunkt aus Schieberegister abgreifen
  //
  assign PIXEL = PIXSR[`Dsz-1];

  //
  // Register schreiben
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0) begin                       // Alle Register auf
      H   <= `H;                               // Default-Werte setzen
      V   <= `V;
      Msz <= (`H)*(`V) / `Bsz;
    end
    else begin
      Msz <= H * V / `Bsz; // Bildgroesse berechnen (/`Bsz wird Verdrahtung!)
      if (WE)                                  // Es wird geschrieben
        case ({PADDR[14],PADDR[2:0]})          // Daten uebernehmen
          1: H <= PDATA;
          5: V <= PDATA;
        endcase
    end
  end

  //
  // Bildspeicheradresse aktualisieren
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0)                    ADDR <= 0;  // Reset
    else
      if ((NXTAD == 1) && (ADRIE == 1)) begin     // neue Adresse
        if (ADDR == Msz-1) ADDR <= 0;             // Ende Bildspeicher
        else               ADDR <= ADDR+1;
      end
  end

  //
  // Bildpunkt-Schieberegister aktualisieren
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0) PIXSR <= 0;                    // Reset
    else
      if ((NXTAD == 1) && (ADRIE == 1))
        PIXSR <= DATA;                            // neues Byte laden
      else PIXSR <= {PIXSR[`Dsz-2:0], 1'b0};      // schieben
  end

endmodule // memacc_ram
