//--------------------------------------------------------------------
//
// Modul vcount_ram - Version mit variabel programmierbarer Aufloesung
//                    und beschreibbarem Videospeicher
//
// Timing des Bildes; Steuersignale fuer Adressen, Bildpunkte und
// vertikale Synchronisation
//
//--------------------------------------------------------------------
//
// Schnittstelle
//
//  ADRIE  Freigabe: Bilddaten uebernehmen und Bildadresse erhoehen
//         (address increment enable) (1-aktiv)
//  VSYNC  vertikale (Bild-)Synchronisation (1-aktiv)
//  NXTLN  im naechsten Takt Zeile hochzaehlen (1-aktiv)
//  CLOCK  Takt
//  NRST   Initialisierung (0-aktiv)
//  PADDR  Programmier-Adressbus
//  DATA   Programmier-Datenbus
//  WE     Write-Enable fuer Programmierbus
//
//--------------------------------------------------------------------
//
// Timing: die Arbeitsperiode von vcount dauert Vframe Zeilen und
// beginnt bei NXTLN=1 mit einer steigenden Taktflanke:
//
// V1 Zeilen  oberer Bildrand
//             VSYNC=0, ADRIE=0
// V  Zeilen  Bildzeilen
//             VSYNC=0, ADRIE=1
// V2 Zeilen  unterer Bildrand
//             VSYNC=0, ADRIE=0
// V3 Zeilen  vertikale Synchronisation
//             VSYNC=1, ADRIE=0
//
//--------------------------------------------------------------------

`include "discount_defs.v"

module vcount_ram (ADRIE, VSYNC, NXTLN, CLOCK, NRST, PADDR, DATA, WE);

  // Ausgaenge
  output ADRIE,      // Freigabe: Bildadresse erhoehen
         VSYNC;      // Bildsynchronisation

  // Eingaenge
  input            NXTLN,      // Zeile hochzaehlen
                   CLOCK,      // Takt
                   NRST,       // Initialisierung (0-aktiv)
                   WE;         // Write-Enable
  input [`Asz-1:0] PADDR;      // Adressbus

  // Bidirektional
  inout [`Rsz-1:0] DATA;       // Datenbus


  // Port-Typen
  reg             ADRIE,
                  VSYNC;
  reg  [`Rsz-1:0] DATA_OUT;
  wire            NXTLN,
                  CLOCK,
                  NRST;
  wire [`Asz-1:0] PADDR;

  // interne Variablen
  reg [`Rsz-1:0] VCNT;    // Zeilenzaehler
  reg [1:0]      VPHASE;  // Phasen vertikal:
                          //   0 oberer Bildrand
                          //   1 Bildbereich
                          //   2 unterer Bildrand
                          //   3 vertikale Synchronisation
  reg [`Rsz-1:0] V1, // Register (variable Aufloesung)
                 V,
                 V2,
                 V3,
                 Vframe;
  reg [`Rsz-1:0] Vframe_temp,  // Temporaere Flip-Flops
                 Vframe_temp2; // fuer Pipeline
  reg            ADDRESSED;    // Gueltige Adresse im vorigen Takt


  //
  // Bidirektionalen Bus asynchron treiben
  // Daten einen Takt verzoegert zu Adressen -> ADDRESSED
  //
  assign DATA = (WE | ~ADDRESSED) ? `Rsz'bz : DATA_OUT;

  //
  // Register synchron auslesen
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0) begin                       // Reset, magic DisCount
      DATA_OUT <= `Rsz'hDC;
      ADDRESSED <= 0;
    end
    else begin
      case (PADDR[2:0])
        4: DATA_OUT <= V1;
        5: DATA_OUT <= V;
        6: DATA_OUT <= V2;
        7: DATA_OUT <= V3;
        default: DATA_OUT <= `Rsz'hBA;         // magic Bad Address
      endcase
      ADDRESSED <= PADDR[2] & ~PADDR[14];
    end
  end

  //
  // Register schreiben
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0) begin                       // Alle Register auf
      V1 <= `V1;                               // Default-Werte setzen
      V  <= `V;
      V2 <= `V2;
      V3 <= `V3;
      Vframe       <= `V1+`V+`V2+`V3;
      Vframe_temp  <= `V1+`V;
      Vframe_temp2 <= `V1+`V+`V2;
    end
    else begin
      Vframe_temp <= V1 + V;                   // Summe in Pipeline
      Vframe_temp2 <= Vframe_temp + V2;        // berechnen
      Vframe <= Vframe_temp2 + V3;
      if (WE)                                  // Es wird geschrieben
        case ({PADDR[14],PADDR[2:0]})          // Daten uebernehmen
          4: V1 <= DATA;
          5: V  <= DATA;
          6: V2 <= DATA;
          7: V3 <= DATA;
        endcase
    end
  end

  //
  // Zeilenzaehler aktualisieren
  // (Periode Vframe, Bereich 0..Vframe-1)
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0)            VCNT <= 0;         // Reset
    else
      if (NXTLN == 1) begin                      // neue Zeile
        if (VCNT == Vframe-1) VCNT <= 0;         // neues Bild
        else                  VCNT <= VCNT+1;
      end
  end

  //
  // Phasenzaehler aktualisieren
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0) VPHASE <= 3;        // Reset
    else
      if (NXTLN == 1)                  // neue Zeile
        case (VCNT)
          0:       VPHASE <= 0;        // oberer Bildrand
          V1:      VPHASE <= VPHASE+1; // Bildbereich
          V1+V:    VPHASE <= VPHASE+1; // unterer Bildrand
          V1+V+V2: VPHASE <= VPHASE+1; // vertikale Synchronisation
        endcase
  end

  //
  // vertikale Synchronisation aktualisieren
  //
  always @(posedge CLOCK or negedge NRST) begin
    if (NRST == 0)     VSYNC <= 1;        // Reset
    else
      if (VPHASE == 3) VSYNC <= 1;
      else             VSYNC <= 0;
  end

  //
  // ADRIE aktualisieren
  //
  always @(VPHASE) begin
    if (VPHASE == 1) ADRIE <= 1;
    else             ADRIE <= 0;
  end

endmodule // vcount_ram
